package geniusweb.bidspace.pareto;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import geniusweb.issuevalue.Bid;
import geniusweb.issuevalue.Domain;
import geniusweb.profile.Profile;
import geniusweb.profile.utilityspace.LinearAdditive;

/**
 * pareto frontier implementation for {@link LinearAdditive}. This is a highly
 * optimized pareto method for {@link LinearAdditive} spaces
 */
public class ParetoLinearAdditive implements ParetoFrontier {

	private final List<LinearAdditive> utilSpaces;
	private Set<Bid> points = null;
	// all issues in a deterministic order

	/**
	 * 
	 * @param utilSpaces. Must contain at least two {@link LinearAdditive}s and
	 *                    all must be defined on the same donain.
	 */
	public ParetoLinearAdditive(List<LinearAdditive> utilSpaces) {
		if (utilSpaces == null || utilSpaces.size() < 2) {
			throw new IllegalArgumentException(
					"utilSpaces must contain at least 2 spaces");
		}
		Domain domain = utilSpaces.get(0).getDomain();
		for (LinearAdditive space : utilSpaces) {
			if (!space.getDomain().equals(domain)) {
				throw new IllegalArgumentException(
						"Expected all spaces using domain " + domain.getName()
								+ " but found " + space.getDomain().getName());
			}
		}
		this.utilSpaces = utilSpaces;
	}

	@Override
	public List<Profile> getProfiles() {
		return Collections.unmodifiableList(utilSpaces);
	}

	@Override
	public Set<Bid> getPoints() {
		if (points == null) {
			points = computePareto();
		}
		return points;
	}

	@Override
	public String toString() {
		return "Pareto " + getPoints();
	}

	private Set<Bid> computePareto() {
		LinkedList<String> issues = new LinkedList<>(
				utilSpaces.get(0).getDomain().getIssues());
		PartialPareto partial = PartialPareto.create(utilSpaces, issues);
		return partial.getPoints().stream().map(point -> point.getBid())
				.collect(Collectors.toSet());
	}

}
